/*
	*	EniButton.java
	*
	*	Written:  21 Sep 1999 by Dean Addison (GSI Canada Ltd.  http://www.gsi.bc.ca)
	*
	*	This program will enumerate all 1-Wire devices on the 1-Wire bus.  It has
	*  a list of descriptions for 1-Wire devices that is by no means complete.
	*  I would appreciate updates if you add to the program.
	*  My e-mail is daddison@gsi.bc.ca.
	* 	
	*	Copyright (C) 1999 GSI Canada Ltd.
	*	
	*	This program is free software; you can redistribute it and/or modify
	*	it under the terms of the GNU General Public License as published by
	*	the Free Software Foundation; either version 2 of the License, or
	*	(at your option) any later version.
	*
	*	See http://www.fsf.org/copyleft/gpl.html for details.
	*	
	*	This program is distributed in the hope that it will be useful,
	*	but WITHOUT ANY WARRANTY; without even the implied warranty of
	*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	*	GNU General Public License for more details.
	*	
	*	You should have received a copy of the GNU General Public License
	*	along with this program (COPYING.TXT); if not, write to the Free Software
	*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.ibutton.*;
import com.dalsemi.comm.*;

/**
 *  This class enumerates 1-Wire devices.
 */
public class EniButton {

//  This array contains the descriptions of the 1-Wire devices.
//  I couldn't figure out how to make this and the next array static.
//  Actually, the familyName is easy but the only convenient way to
//	 initialise idxFamilyName is at runtime, so I made them both non-static.
	public final String[] familyName = {
		"Unknown",
		"DS1990A Serial Number iButton",
		"DS1991 Multikey iButton",
		"DS1994 Memory Plus Time iButton 4kb NVRAM",
		"DS1993 Memory iButton 4kb NVRAM",
		"DS1992 Memory iButton 1kb NVRAM",
		"DS1982 Add-Only iButton 1kb OTP EPROM",
		"DS1995 Memory iButton 16kb NVRAM",
		"DS1985 Add-Only iButton 16kb OTP EPROM",
		"DS1996 Memory iButton 64kb NVRAM",
		"DS1986 Add-Only iButton 64kb OTP EPROM",
		"DS1920 Temperature iButton",
		"DS1921 Thermochron iButton",
		"DS1973 4K EEPROM iButton",
		"DS2407 (aka DS2405A) Addressable Switch 1kb EPROM",  // I don't have the family codes for these.
		"DS2404S-C01 Dual Port RAM Plus Time EEPROM iButton" };

/* Typically, you can add the descriptions of the 1-Wire devices at the end
	of the familyName array and then point to them with the idxFamilyName array.
	The index into the familyName array is stored in the idxFamilyName array at
	the position equal to the family code.  It acts as a simplistic look up table.
	This is not the optimum way of doing this, just a quick solution.
	There are a number of ways to modify this to use a more compact representation.
*/
	public int[] idxFamilyName = new int[0xFF];  // We set this up in the constructor.

	// The current iWire device in the iteration
	private Access OWDevice;

	// The 8 bytes of rom data
	int[] romData;

	/**
	* Construct a new instance of OWDevice.
	* Initialise the idxFamilyName array.
	*/
	public EniButton() {
		try {

			int i = 0;
			
			for(i = 0 ; i < idxFamilyName.length ; i++)
				idxFamilyName[i] = 0;  // The default:  unknown.

			idxFamilyName[0x01] = 1;  // 0x01 == Serial Number iButton.
			idxFamilyName[0x02] = 2;
			idxFamilyName[0x04] = 3;
			idxFamilyName[0x06] = 4;
			idxFamilyName[0x08] = 5;
			idxFamilyName[0x09] = 6;
			idxFamilyName[0x0A] = 7;
			idxFamilyName[0x0B] = 8;
			idxFamilyName[0x0C] = 9;
			idxFamilyName[0x0F] = 10;
			idxFamilyName[0x10] = 11;  // This is the Temperature iButton (aka DS1820).
			idxFamilyName[0x21] = 12;  // Here is the Thermochron iButton.
			idxFamilyName[0x23] = 13;

			OWDevice  = new Access();

			// Change this to whatever port you are using.  It should work anywhere.
			OWDevice.SetAdapterType(SerialPort.SERIAL0, "port0");

		}
		catch(Exception e){}
	}
	/** Find the first device on the 1-Wire bus.
	*/
	public boolean getFirstDevice() {

		romData = null;
		boolean found = false;

		if(OWDevice.iBReset())  // clear the line.
			if(OWDevice.iBFirst()) {  // get first?
				romData   = OWDevice.iBROMData();
				found = true;
			}

		return found;
	}
	/** Find the next device on the 1-Wire bus.
	*/
	public boolean getNextDevice() {

		romData = null;
		boolean found = false;

		if(OWDevice.iBReset())  // clear the line
			if(OWDevice.iBNext()) {  // get next
				romData   = OWDevice.iBROMData();
				found = true;
			}

		return found;
	}
	public String getDeviceInfo(int num) {

		int i = 0;
		String info = "";
		
		//  Start the string.  This bit should really be in
		//  main(), then we wouldn't need num passed in.
		info = "1Wire["+Integer.toString(num)+"]=0x";

		//  Add the 48-bit ROM ID.
		for(i = romData.length-2 ; i > 0 ; i--) {
			if(romData[i] <= 0x0F)
				info += "0";		// I wish I knew a better way
			if(romData[i] == 0)	// to add two digit Hexadecimal.
				info += "0";
			else
				info += Integer.toHexString(romData[i]).toUpperCase();
		}
			
		//  The family code for the device.
		info += " Family=0x";
		if(romData[0] <= 0x0F)
			info += "0";
		if(romData[0] == 0)
			info += "0";
		else
			info += Integer.toHexString(romData[0]).toUpperCase();

		//  The CRC code.  Not that anybody cares for this app.
		info += " CRC=0x";
		if(romData[romData.length-1] <= 0x0F)
			info += "0";
		if(romData[romData.length-1] == 0)
			info += "0 <-- !!!";  // Something is seriously wrong if this happens.
		else
			info += Integer.toHexString(romData[romData.length-1]).toUpperCase();


		//  Here's the fun stuff.  Add the description.
		if((romData[0] > 0x0) & (romData[0] < 0xFF))
			//  The family code indexes the familyName strings.
			info += " ("+familyName[idxFamilyName[romData[0]]]+").";

		return info;
	}

	public static void main(String[] args) {

		System.out.println("Enumerating iButtons..."); // I.e. 1-Wire Devices.
		try {
	
			int count = 0;
			EniButton button = new EniButton();

			if(button.getFirstDevice())
				System.out.println(button.getDeviceInfo(count++));
			
			while(button.getNextDevice())
				System.out.println(button.getDeviceInfo(count++));

			if(count == 1)
				System.out.println("Total of "+Integer.toString(count)+" iButton.");
			else
				System.out.println("Total of "+Integer.toString(count)+" iButtons.");
	
		}
		catch(Throwable t) {
			System.out.println(t);
			System.out.println(t.getMessage());      
		}
	}
}
	

